#include "StdAfx.h"
#include "Scene.h"

CScene::CScene(void) {
}

CScene::~CScene(void) {
	Nodes.clear();
}

void CScene::Activate(void) {
	Current = this;
}

CScene * CScene::Current = NULL;

CScene * CScene::GetCurrent(void) {
	return Current;
}

void CScene::AddNode(CNode * node) {
	Nodes.push_back(node);
	node->Scene = this;
	node->Init();
}

void CScene::Init(void) {
	int w, h;
	glfwGetWindowSize(&w, &h);
	Camera.SetViewport(w, h);

	SP_General = new CShadingProgram();
	SP_General->AttachShader(GL_VERTEX_SHADER, "Shaders/General.vert");
	SP_General->AttachShader(GL_FRAGMENT_SHADER, "Shaders/Phong.frag");
}

void CScene::Redraw(void) {
	Time = CApplication::GetInstance()->GetTime();
	for (std::vector<CNode *>::iterator it = Nodes.begin(); it != Nodes.end(); ++it) {
		CRenderableNode * rNode = dynamic_cast<CRenderableNode *>(*it);
		if (rNode != NULL) {
			rNode->Draw();
		}
	}
}

void CScene::Update(void) {
	Time = CApplication::GetInstance()->GetTime();
	for (std::vector<CNode *>::iterator it = Nodes.begin(); it != Nodes.end(); ++it) {
		(*it)->Update();
	}
}

void CScene::BindLights(void) {
	int i = 0;
	for (std::vector<CLight *>::iterator it = Lights.begin(); it != Lights.end(); ++it) {
		(*it)->Bind(i++);
	}
	CLight::SetLightsNumber(Lights.size());
}
