#version 330 core

in VertexData {
	vec3 v3Normal;
	vec3 v3Eye;
} VertexIn;

layout(location = 0) out vec4 v4Color;

struct LightData {
	vec3 v3Position;
	vec3 v3Ambient;
	vec3 v3Diffuse;
	vec3 v3Specular;
};

struct MaterialData {
	vec3 v3Ambient;
	vec3 v3Diffuse;
	vec3 v3Specular;
	float fShininess;
};

uniform MaterialData mdMaterial;
uniform int iLights;
uniform LightData ldLights[8];

void main(void) {
	vec3 v3Normal = normalize(VertexIn.v3Normal);
	vec3 v3Color;
	for (int i = 0; i < iLights; ++i) {
		vec3 v3Direction = normalize(ldLights[i].v3Position - VertexIn.v3Eye);
		float nDotL = max(dot(v3Direction, v3Normal), 0.0);
		vec3 v3Reflected = reflect(-v3Direction, v3Normal);
		float eDotR = max(dot(-normalize(VertexIn.v3Eye), v3Reflected), 0.0);
		v3Color += ldLights[i].v3Ambient * mdMaterial.v3Ambient + ldLights[i].v3Diffuse * nDotL * mdMaterial.v3Diffuse + ldLights[i].v3Specular * pow(eDotR, mdMaterial.fShininess) * mdMaterial.v3Specular;
	}
	v4Color = vec4(v3Color, 1);
}
