#include "StdAfx.h"
#include "ShadingProgram.h"


CShadingProgram::CShadingProgram(void)
{
	Id = glCreateProgram();
	isLinked = false;
	isVertexShaderAttached = false;
	isFragmentShaderAttached = false;
}


CShadingProgram::~CShadingProgram(void)
{
}

void CShadingProgram::Activate(void) {
	Current = this;
	glUseProgram(Id);
	if (!isLinked) {
		Link();
	}
}

void CShadingProgram::Deactivate(void) {
	Current = NULL;
	glUseProgram(0);
}

void CShadingProgram::AttachShader(GLenum type, const char *path) {
	ifstream shaderFile;
	string * shaderCodeStr;
	
	vector<string> lines;
	shaderFile.open(path, ifstream::in);
	shaderCodeStr = new string(istreambuf_iterator<char>(shaderFile), istreambuf_iterator<char>());
	shaderFile.close();

	*shaderCodeStr += "\0";

	const char * shaderCode = shaderCodeStr->c_str();

	GLuint shaderId = glCreateShader(type);
	glShaderSource(shaderId, 1, &shaderCode, NULL);
	glCompileShader(shaderId);

	int status;
	glGetShaderiv(shaderId, GL_COMPILE_STATUS, &status);
	if (status == GL_FALSE) {
		cerr << "Shader compilation failed (" << path << ").\n";
		GLchar *buf = new GLchar[512];
		glGetShaderInfoLog(shaderId, 512, NULL, buf);
		cerr << buf << "\n";
		delete[] buf;
		glDeleteShader(shaderId);
		return;
	}

	glUseProgram(Id);
	glAttachShader(Id, shaderId);

	delete shaderCode;

	switch (type) {
		case GL_VERTEX_SHADER:
			isVertexShaderAttached = true;
			break;
		case GL_FRAGMENT_SHADER:
			isFragmentShaderAttached = true;
			break;
	}
	shaderIds.push_back(shaderId);
}

GLint CShadingProgram::GetUniformLocation(const char *name) {
	return glGetUniformLocation(Id, name);
}

void CShadingProgram::Link(void) {
	if (!isVertexShaderAttached) {
		cerr << "Cannot link a shading program without a vertex shader!\n";
	}
	if (!isFragmentShaderAttached) {
		cerr << "Cannot link a shading program without a fragment shader!\n";
	}

	glLinkProgram(Id);

	int status;
	glGetProgramiv(Id, GL_LINK_STATUS, &status);
	if (status == GL_FALSE) {
		cerr << "Program linkage failed (" << Id << ").\n";
		GLchar *buf = new GLchar[512];
		glGetProgramInfoLog(Id, 512, NULL, buf);
		cerr << buf << "\n";
		delete[] buf;
		return;
	}

	isLinked = true;

	for (vector<GLuint>::iterator it = shaderIds.begin(); it != shaderIds.end(); ++it) {
		glDetachShader(Id, *it);
		glDeleteShader(*it);
	}
}

CShadingProgram * CShadingProgram::Current = NULL;

CShadingProgram * CShadingProgram::GetCurrent(void) {
	return Current;
}
